package analyzer

import "fmt"
import "github.com/alecthomas/participle/v2"
import "github.com/alecthomas/participle/v2/lexer"
import "git.tebibyte.media/sashakoshka/fspl/entity"

func (this *Tree) canAssignStrict (
	pos  lexer.Position,
	// assuming both are analyzed already
	destination entity.Type,
	source      entity.Type,
) error {
	can := false
	switch destination.(type) {
	// named type
	case *entity.TypeNamed:
		// try to compare names first
		destination := destination.(*entity.TypeNamed)
		if source, ok := source.(*entity.TypeNamed); ok {
			if destination.Name == source.Name {
				can = true
				break
			}
		}

		// if not, types may be assignmable further down:
		return this.canAssignStrict(pos, destination.Type, source)

	// interface tyoe
	case *entity.TypeInterface:
		destination := destination.(*entity.TypeInterface)
		for name, behavior := range destination.BehaviorMap {
			// get method
			method := this.methodOrBehaviorOf(source, name)
			if method == nil {
				return participle.Errorf (
					pos, "%v is missing method %v",
					source, name)
			}
			
			// extract signature
			var signature *entity.Signature
			switch method.(type) {
			case *entity.Signature:
				signature = method.(*entity.Signature)
			case *entity.Method:
				signature = method.(*entity.Method).Signature
			default:
				panic(fmt.Sprint (
					"Tree.methodOrBehaviorOf returned ",
					method))
			}

			// check equivalence
			if !signature.Equals(behavior) {
				return participle.Errorf (
					pos, "%v has wrong signature for method %v",
					source, name)
			}
		}
		
	// if ty is not named nor an interface, we can test for structural
	// equivalence:
	case *entity.TypeVoid,
		*entity.TypePointer,
		*entity.TypeSlice,
		*entity.TypeArray,
		*entity.TypeStruct,
		*entity.TypeInt,
		*entity.TypeFloat,
		*entity.TypeWord:

		can = destination.Equals(source)
		
	default: panic(fmt.Sprint("BUG: analyzer doesnt know about type ", destination))
	}

	if can {
		return nil
	} else {
		return participle.Errorf(pos, "expected %v", destination)
	}
}
