package analyzer

import "testing"

func TestAssignmentLiteralErrUnsignedNegative (test *testing.T) {
testStringErr (test,
"expected unsigned integer", 3, 11,
`
[main] = {
	x:UInt = -5
}
`)
}

func TestAssignmentLiteralErrIntegerFloat (test *testing.T) {
testStringErr (test,
"expected integer", 3, 10,
`
[main] = {
	x:Int = 5.5
}
`)
}

func TestAssignmentLiteralErrIntegerStruct (test *testing.T) {
testStringErr (test,
"expected integer", 3, 10,
`
[main] = {
	x:Int = (x: 5)
}
`)
}

func TestAssignmentLiteralErrIntegerArray (test *testing.T) {
testStringErr (test,
"expected integer", 3, 10,
`
[main] = {
	x:Int = (* 5)
}
`)
}

func TestAssignmentLiteralErrIntegerOverflow (test *testing.T) {
testStringErr (test,
"expected smaller number", 3, 11,
`
[main] = {
	x:Byte = 1000
}
`)
}

func TestAssignmentLiteralErrArrayInt (test *testing.T) {
testStringErr (test,
"expected array", 3, 11,
`
[main] = {
	x:3:Int = 5
}
`)
}

func TestAssignmentLiteralErrArrayWrongLength (test *testing.T) {
testStringErr (test,
"expected 3 elements", 3, 11,
`
[main] = {
	x:3:Int = (* 1 2 3 4)
}
`)
}

func TestAssignmentLiteralErrArrayWrongType (test *testing.T) {
testStringErr (test,
"expected integer", 3, 19,
`
[main] = {
	x:3:Int = (* 1 2 3.3 4)
}
`)
}

func TestAssignmentLiteralErrSliceWrongType (test *testing.T) {
testStringErr (test,
"expected integer", 3, 19,
`
[main] = {
	x:*:Int = (* 1 2 3.3 4)
}
`)
}

func TestAssignmentLiteralErrStructWrongType (test *testing.T) {
testStringErr (test,
"expected integer", 3, 26,
`
[main] = {
	x:3:(x:Int y:Int) = (x: 5.5 y: 3)
}
`)
}
func TestAssignmentLiteralErrStructInteger (test *testing.T) {
testStringErr (test,
"expected struct", 3, 26,
`
[main] = {
	x:3:(x:Int y:Int) = 5
}
`)
}

func TestAssignmentLiteralErrInterfaceInt (test *testing.T) {
testStringErr (test,
"cannot assign literal to interface", 6, 2,
`
Bird: ([fly distance:F64] [land])
[main] = {
	b:Bird = 5
}
`)
}

func TestAssignmentLiteralErrInterfaceFloat (test *testing.T) {
testStringErr (test,
"cannot assign literal to interface", 6, 2,
`
Bird: ([fly distance:F64] [land])
[main] = {
	b:Bird = 5.5
}
`)
}

func TestAssignmentLiteralErrInterfaceArray (test *testing.T) {
testStringErr (test,
"cannot assign literal to interface", 6, 2,
`
Bird: ([fly distance:F64] [land])
[main] = {
	b:Bird = (* 1 2 3 4)
}
`)
}

func TestAssignmentLiteralErrInterfaceStruct (test *testing.T) {
testStringErr (test,
"cannot assign literal to interface", 6, 2,
`
Bird: ([fly distance:F64] [land])
BlueJay: Int
BlueJay::[land] = { }
[main] = {
	b:Bird = (x: 5 y: 6)
}
`)
}

func TestAssignmentLiteral (test *testing.T) {
testString (test,
`
[main] = {
	a:F64 = 5.3
	b:F32 = 5.3
	c:F64 = -5
	d:F32 = -5
	e:Byte = 64
	z:UInt = 5
	x:Int  = -5
	arr:4:2:Int = (*
		(* 1 2)
		(* 3 4)
		(* 5 6)
		(* 7 8))
	slice:*:Int = (* 3 1 2 3)
	struct:(x:Int y:Int) = (
		x: 9
		y: 10)
}
`)
}

func TestAssignmentInterfaceErrBadSignature (test *testing.T) {
testStringErr (test,
"BlueJay has wrong signature for method fly", 7, 2,
`
Bird: ([fly distance:F64] [land])
BlueJay: Int
BlueJay::[fly] = { }
BlueJay::[land] = { }
[main] = {
	b:Bird = [@a:BlueJay]
}
`)
}

func TestAssignmentInterface (test *testing.T) {
testString (test,
`
Bird: ([fly distance:F64] [land])
BlueJay: Int
BlueJay::[fly distance:F64] = { }
BlueJay::[land] = { }
[main] = {
	a:BlueJay
	b:Bird = [@a]
}
`)
}

// TODO: complete and test error cases
func TestAssignmentPropagateRules (test *testing.T) {
testString (test,
`
[f]:Byte = 5
A:Int
A::[g]:Int = 2
B:([g]:Int)
[main] = {
	a:Int
	b:Int = a
	c:Int = d:Int
	d:Int = { a:F64 b }
	e:Byte = [f]
	g:(x:Int y:(w:F64 z:F64)) = (x: 1 y: (w: 1.2 z: 78.5))
	gx:(w:F64 z:F64) = g.x
	h:F64 = gx.z
	i:A
	j:Int = [i::g]
	k:B = i
	l:Int = [k::g]
	m:5:Int = (* 0 1 2 3 4)
	n:Int = [. m 3]
}
`)
}
