package analyzer

import "testing"

func TestCastErrIntPointer (test *testing.T) {
testStringErr (test,
"cannot convert from *Int to Int", 2, 14,
`
[main]:Int = [~ Int [@ a:Int]]
`)
}

func TestCastErrIntStruct (test *testing.T) {
testStringErr (test,
"cannot convert from (x:Int y:Int) to Int", 2, 14,
`
[main]:Int = [~ Int a:(x:Int y:Int)]
`)
}

func TestCastErrIntArray (test *testing.T) {
testStringErr (test,
"cannot convert from 5:Int to Int", 2, 14,
`
[main]:Int = [~ Int a:5:Int]
`)
}

func TestCastErrIntSlice (test *testing.T) {
testStringErr (test,
"cannot convert from *:Int to Int", 2, 14,
`
[main]:Int = [~ Int a:*:Int]
`)
}

func TestCastErrPointerInt (test *testing.T) {
testStringErr (test,
"cannot convert from Int to *Int", 2, 15,
`
[main]:*Int = [~ *Int [@ a:Int]]
`)
}

func TestCastErrStructInt (test *testing.T) {
testStringErr (test,
"cannot convert from Int to (x:Int y:Int)", 2, 24,
`
[main]:(x:Int y:Int) = [~ (x:Int y:Int) a:Int]
`)
}

func TestCastErrArrayInt (test *testing.T) {
testStringErr (test,
"cannot convert from Int to 5:Int", 2, 13,
`
[main]:Int = [~ Int a:5:Int]
`)
}

func TestCastErrSliceInt (test *testing.T) {
testStringErr (test,
"cannot convert from Int to *:Int", 2, 16,
`
[main]:*:Int = [~ *:Int a:Int]
`)
}

func TestCast (test *testing.T) {
testString (test,
`
Bird: ([fly distance:F64] [land])
BlueJay: Int
BlueJay::[fly distance:F64] = { }
BlueJay::[land] = { }
IntDerived: Int
[main] = {
	a:IntDerived = 5
	b:Int [~ Int [~ F64 [~ Byte a]]]
	c:Int [~~ Int [~~ F64 [~~ Byte a]]]
	d:(x:Int y:Int) = (x: 1 y: 2)
	e:(z:Int a:Int) = [~~ (z:Int a:Int) d]
	f:Bird = [@ [~~ BlueJay 0]]
	g:*:Int = [~ *:Int h:5:Int]
}
`)
}
