package analyzer

import "fmt"
import "git.tebibyte.media/sashakoshka/fspl/entity"

func (this *Tree) analyzeExpression (
	into       entity.Type,
	expression entity.Expression,
) (
	entity.Expression,
	error,
) {
	switch expression.(type) {
	case *entity.Variable:
		return this.analyzeVariable(into, expression.(*entity.Variable))
	case *entity.Declaration:
		return this.analyzeDeclaration(into, expression.(*entity.Declaration))
	case *entity.Call:
		return this.analyzeCall(into, expression.(*entity.Call))
	case *entity.MethodCall:
		return this.analyzeMethodCall(into, expression.(*entity.MethodCall))
	// case *entity.Subscript:
	// 	return this.analyzeSubscript(into, expression.(*entity.Subscript))
	// case *entity.Slice:
	// 	return this.analyzeSlice(into, expression.(*entity.Slice))
	// case *entity.Dereference:
	// 	return this.analyzeDereference(into, expression.(*entity.Dereference))
	// case *entity.Reference:
	// 	return this.analyzeReference(into, expression.(*entity.Reference))
	// case *entity.ValueCast:
	// 	return this.analyzeValueCast(into, expression.(*entity.ValueCast))
	// case *entity.BitCast:
	// 	return this.analyzeBitCast(into, expression.(*entity.BitCast))
	// case *entity.Operation:
	// 	return this.analyzeOperation(into, expression.(*entity.Operation))
	// case *entity.Block:
	// 	return this.analyzeBlock(into, expression.(*entity.Block))
	// case *entity.MemberAccess:
	// 	return this.analyzeMemberAccess(into, expression.(*entity.MemberAccess))
	// case *entity.IfElse:
	// 	return this.analyzeIfElse(into, expression.(*entity.IfElse))
	// case *entity.Loop:
	// 	return this.analyzeLoop(into, expression.(*entity.Loop))
	// case *entity.Break:
	// 	return this.analyzeBreak(into, expression.(*entity.Break))
	// case *entity.Return:
	// 	return this.analyzeReturn(into, expression.(*entity.Return))
	default:
		panic(fmt.Sprint (
			"BUG: analyzer doesnt know about expression ",
			expression))
	}
}

func (this *Tree) analyzeStatement (
	statement entity.Statement,
) (
	entity.Statement,
	error,
) {
	if assignment, ok := statement.(*entity.Assignment); ok {
		return this.analyzeAssignment(assignment)
	} else if expression, ok := statement.(entity.Expression); ok {
		expression, err := this.analyzeExpression(nil, expression)
		return expression.(entity.Statement), err
	} else {
		panic(fmt.Sprint (
			"BUG: analyzer doesnt know about statement ",
			statement))
	}
}

