package analyzer

import "github.com/alecthomas/participle/v2"
import "github.com/alecthomas/participle/v2/lexer"
import "git.tebibyte.media/sashakoshka/fspl/entity"

func (this *Tree) analyzeFunction (
	pos lexer.Position,
	name string,
) (
	*entity.Function,
	error,
) {
	var err error
	
	// return if exists already
	if function, exists := this.Functions[name]; exists {
		return function, nil
	}
	
	// error if function is missing
	function, exists := this.rawFunctions[name]
	if !exists {
		return nil, participle.Errorf(pos, "no function named %s", name)
	}

	// create a new scope context for this function
	this.PushScopeContext()
	this.PushScope(function)
	defer this.PopScopeContext()
	defer this.PopScope()
	
	// analyze signature and add arguments to root scope of function
	function.Signature, err = this.assembleSignatureMap(function.Signature)
	if err != nil { return function, err }
	for name, argument := range function.Signature.ArgumentMap {
		argument.Type, err = this.analyzeType(argument.Type, false)
		this.AddVariable(argument)
		function.Signature.ArgumentMap[name] = argument
		if err != nil { return function, err }
	}
	function.Signature.Return, err =
		this.analyzeType(function.Signature.Return, false)
	if err != nil { return function, err }

	// add incomplete function to complete functions because there is enough
	// information for it to be complete from the point of view of other
	// parts of the code
	this.Functions[name] = function

	// TODO: analyze function body
	return function, nil
}
