package analyzer

import "github.com/alecthomas/participle/v2"
import "github.com/alecthomas/participle/v2/lexer"
import "git.tebibyte.media/sashakoshka/fspl/entity"

func (this *Tree) analyzeMethod (
	pos lexer.Position,
	typeName string,
	name string,
) (
	*entity.Method,
	error,
) {
	// get parent typedef
	owner, err := this.analyzeTypedef(pos, typeName, false)
	if err != nil { return nil, err }
	
	// return if exists already
	if method, exists := owner.Methods[name]; exists {
		return method, nil
	}
	
	// error if method is missing
	method, exists := this.rawMethods[typeName + "." + name]
	if !exists {
		return nil, participle.Errorf (
			pos, "no method named %s.%s",
			typeName, name)
	}

	// create a new scope context for this function
	this.PushScopeContext()
	this.PushScope(method)
	defer this.PopScopeContext()
	defer this.PopScope()
	
	// analyze signature and add arguments to root scope of function
	method.Signature, err = this.assembleSignatureMap(method.Signature)
	if err != nil { return method, err }
	for name, argument := range method.Signature.ArgumentMap {
		argument.Type, err = this.analyzeType(argument.Type, false)
		this.AddVariable(argument)
		method.Signature.ArgumentMap[name] = argument
		if err != nil { return method, err }
	}
	method.Signature.Return, err =
		this.analyzeType(method.Signature.Return, false)
	if err != nil { return method, err }

	// add incomplete method to complete methods because there is enough
	// information for it to be complete from the point of view of other
	// parts of the code
	owner.Methods[name] = method

	// TODO: analyze method body
	return method, nil
}
