package analyzer

import "git.tebibyte.media/sashakoshka/fspl/entity"

// scopeContextManager is a stack of scopeContexts allowing multiple stacks of
// scopes to be managed at the same time in case the analysis of one scoped
// entity causes the analysis of another.
type scopeContextManager []scopeContext

func (this *scopeContextManager) PushScopeContext () {
	*this = append(*this, nil)
}

func (this *scopeContextManager) PopScopeContext () {
	this.assertPopulated()
	*this = (*this)[:len(*this) - 1]
}

func (this *scopeContextManager) PushScope (scope entity.Scoped) {
	this.assertPopulated()
	(*this)[len(*this) - 1].PushScope(scope)
}

func (this *scopeContextManager) PopScope () {
	this.assertPopulated()
	(*this)[len(*this) - 1].PopScope()
}

func (this *scopeContextManager) TopScope () entity.Scoped {
	this.assertPopulated()
	(*this)[len(*this) - 1].TopScope()
}

func (this *scopeContextManager) Variable (name string) *entity.Declaration {
	this.assertPopulated()
	return (*this)[len(*this) - 1].TopScope().Variable(name)
}

func (this *scopeContextManager) AddVariable (declaration *entity.Declaration) {
	this.assertPopulated()
	(*this)[len(*this) - 1].TopScope().AddVariable(declaration)
}

func (this *scopeContextManager) assertPopulated () {
	if len(*this) < 1 {
		panic("scopeContextManager must have at least 1 scope context")
	}
}

// scopeContext is a stack of scopes used when analyzing scoped entity.
type scopeContext []entity.Scoped

func (this *scopeContext) PushScope (scope entity.Scoped) {
	*this = append(*this, scope)
}

func (this *scopeContext) PopScope () {
	this.assertPopulated()
	*this = (*this)[:len(*this) - 1]
}

func (this *scopeContext) TopScope () entity.Scoped {
	this.assertPopulated()
	return (*this)[len(*this) - 1]
}

func (this *scopeContext) Variable (name string) *entity.Declaration {
	this.assertPopulated()
	for index := len(*this) - 1; index >= 0; index -- {
		variable := (*this)[index].Variable(name)
		if variable != nil {
			return variable
		}
	}
	return nil
}

func (this *scopeContext) AddVariable (declaration *entity.Declaration) {
	this.assertPopulated()
	(*this)[len(*this) - 1].AddVariable(declaration)
}

func (this *scopeContext) assertPopulated () {
	if len(*this) < 1 {
		panic("scopeContext must have at least 1 scope")
	}
}
