package parser

import "io"
import "os"
import "fmt"
import "github.com/alecthomas/participle/v2"
import "git.tebibyte.media/sashakoshka/fspl/entity"
import flexer "git.tebibyte.media/sashakoshka/fspl/lexer"

var parser = participle.MustBuild[Tree] (
	participle.Union[entity.TopLevel] (
		&entity.Function { },
		&entity.Typedef  { },
		&entity.Method   { }),
	participle.Union[entity.Type] (
		&entity.TypeNamed     { },
		&entity.TypePointer   { },
		&entity.TypeInterface { },
		&entity.TypeStruct    { },
		&entity.TypeArray     { },
		&entity.TypeSlice     { }),
	participle.Union[entity.Expression] (
		&entity.LiteralInt     { },
		&entity.LiteralFloat   { },
		&entity.LiteralArray   { },
		&entity.LiteralStruct  { },
		&entity.LiteralBoolean { },
		&entity.Break          { },
		&entity.Return         { },
		&entity.IfElse         { },
		&entity.Loop           { },
		&entity.MethodCall     { },
		&entity.MemberAccess   { },
		&entity.Declaration    { },
		&entity.Variable       { },
		&entity.Subscript      { },
		&entity.Slice          { },
		&entity.Dereference    { },
		&entity.Reference      { },
		&entity.ValueCast      { },
		&entity.BitCast        { },
		&entity.Operation      { },
		&entity.Call           { },
		&entity.Block          { }),
	participle.Union[entity.Statement] (
		&entity.LiteralInt     { },
		&entity.LiteralFloat   { },
		&entity.LiteralArray   { },
		&entity.LiteralStruct  { },
		&entity.LiteralBoolean { },
		&entity.Break          { },
		&entity.Return         { },
		&entity.Assignment     { },
		&entity.IfElse         { },
		&entity.Loop           { },
		&entity.MethodCall     { },
		&entity.MemberAccess   { },
		&entity.Declaration    { },
		&entity.Variable       { },
		&entity.Subscript      { },
		&entity.Slice          { },
		&entity.Dereference    { },
		&entity.Reference      { },
		&entity.ValueCast      { },
		&entity.BitCast        { },
		&entity.Operation      { },
		&entity.Call           { },
		&entity.Block          { }),
	participle.UseLookahead(participle.MaxLookahead),
	participle.Lexer(flexer.NewDefinition()))

// Tree represents a parsed abstract syntax tree. It has no constructor and its
// zero value can be used safely.
type Tree struct {
	Declarations []entity.TopLevel `parser:" @@* "`
}

func (this *Tree) String () string {
	out := ""
	for index, declaration := range this.Declarations {
		if index > 0 { out += "\n" }
		out += fmt.Sprint(declaration)
	}
	return out
}

// Parse parses the contents of the given io.Reader into the tree.
func (this *Tree) Parse (name string, file io.Reader) error {
	parsed, err := parser.Parse(name, file)
	if parsed != nil {
		this.Declarations = append(this.Declarations, parsed.Declarations...)
	}
	return err
}

// ParseFile parses the contents of the given file into the tree.
func (this *Tree) ParseFile (name string) error {
	file, err := os.Open(name)
	if err != nil { return err }
	defer file.Close()
	return this.Parse(name, file)
}
